<?php

namespace Stachethemes\Stec;

class Builder {

    private static $accepted_posts = array('stec_esf', 'stec_sp');

    public static function delete_project($id) {

        if (false === current_user_can('edit_posts')) {
            throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
        }

        if (!in_array(get_post_type($id), self::$accepted_posts)) {
            throw new Stec_Exception(esc_html__('Invalid post type', 'stec'));
        }

        $result = wp_delete_post((int) $id, true);

        if (false === $result || null === $result) {
            throw new Stec_Exception(esc_html__('Error deleting post', 'stec'));
        }

        return true;
    }

    public static function bulk_delete_projects($ids) {

        if (empty($ids)) {
            return true;
        }

        if (false === current_user_can('edit_posts')) {
            throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
        }

        global $wpdb;

        $filter_ids = filter_var_array($ids, FILTER_VALIDATE_INT);
        $unique_ids = array_filter($filter_ids);

        $placeholders = array_fill(0, count($unique_ids), '%d');
        $format       = implode(', ', $placeholders);

        $post_types = "'" . implode("','", self::$accepted_posts) . "'";

        $sql = $wpdb->prepare("DELETE `POSTS`, `META`, `TERM`  
                FROM {$wpdb->prefix}posts as POSTS
                LEFT JOIN {$wpdb->prefix}postmeta AS META 
                ON META.post_id = POSTS.ID
                LEFT JOIN {$wpdb->prefix}term_relationships AS TERM 
                ON TERM.object_id = POSTS.ID
                WHERE POSTS.post_type IN ($post_types) AND POSTS.ID IN ($format)", $unique_ids);

        $result = $wpdb->query($sql);

        if (is_wp_error($result)) {
            throw new Stec_Exception($result->get_error_message());
        }

        return true;
    }

    public static function get_projects_list_array($project_type) {

        $projects_array = array();
        $projects       = self::get_projects_list($project_type);

        if (isset($projects[$project_type]) && $projects[$project_type]) {

            foreach ($projects[$project_type] as $project) {
                $projects_array[$project['id']] = $project['title'];
            }
        }

        return $projects_array;
    }

    /**
     * Retrieves builder projects list
     * @param mixed $project_type String or Array
     * @return array 
     */
    public static function get_projects_list($project_type = null) {

        if (!$project_type) {
            $project_type_array = self::$accepted_posts;
        }

        if (is_array($project_type)) {
            $project_type_array = $project_type;
        }

        if (isset($project_type_array) && is_array($project_type_array)) {
            $projects_array = array();
            foreach ($project_type_array as $project_type) {
                $projects       = self::get_projects_list($project_type);
                $projects_array = array_merge($projects_array, $projects);
            }
            return $projects_array;
        }

        $accepted_posts = self::$accepted_posts;

        if (!in_array($project_type, $accepted_posts)) {
            return array($project_type => array());
        }

        $posts = get_posts(array(
            'post_type'        => $project_type,
            'fields'           => 'ids',
            'suppress_filters' => 0
        ));

        $projects = array();

        foreach ($posts as $id) {
            $projects[] = array(
                'id'    => $id,
                'title' => get_the_title($id),
                'edit'  => get_edit_post_link($id)
            );
        }

        return array(
            $project_type => $projects
        );
    }

    public static function get_project_content($id) {

        if (0 !== (int) $id && class_exists('\Elementor\Plugin')) {
            $document = \Elementor\Plugin::$instance->documents->get_doc_for_frontend($id);

            if (false === $document) {
                throw new Stec_Exception(esc_html__('Project not found', 'stec'), 404);
            }

            return $document->get_content();
        } else {
            throw new Stec_Exception(esc_html__('Elementor plugin not found', 'stec'), 404);
        }
    }

}
