<?php

namespace Stachethemes\Stec;

class Shop {

    protected static $products      = false;
    protected static $filters_added = false;

    public static function add_shop_filters() {

        if (true === self::$filters_added) {
            return;
        }

        $relevant_shop_meta_keys = array(
            'event_title' => esc_html__('Event Title', 'stec'),
            'start_date'  => esc_html__('Start Date', 'stec'),
            'permalink'   => esc_html__('Permalink', 'stec')
        );

        /**
         * Attach custom meta
         */
        add_filter('woocommerce_add_cart_item_data', function ($cart_item_data, $product_id, $variation_id) use ($relevant_shop_meta_keys) {

            $add_to_cart_post_data = Admin_Helper::post('data', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if (isset($add_to_cart_post_data['stec_shop_item'])) {
                foreach ($relevant_shop_meta_keys as $meta_key => $meta_label) {
                    if (isset($add_to_cart_post_data['stec_shop_item'][$meta_key])) {
                        $cart_item_data[$meta_key] = $add_to_cart_post_data['stec_shop_item'][$meta_key];
                    }
                }
            }

            return $cart_item_data;
        }, 10, 3);

        /**
         * Meta on front
         */
        add_filter('woocommerce_get_item_data', function($item_data, $cart_item) use ($relevant_shop_meta_keys) {
            foreach ($relevant_shop_meta_keys as $meta_key => $meta_label) {
                if (array_key_exists($meta_key, $cart_item)) {
                    $item_data[] = array(
                        'key'   => $meta_label,
                        'value' => $cart_item[$meta_key]
                    );
                }
            }
            return $item_data;
        }, 10, 2);

        /**
         * Meta on order created and dashboard
         */
        add_action('woocommerce_checkout_create_order_line_item', function ($item, $cart_item_key, $values, $order) use ($relevant_shop_meta_keys) {
            foreach ($relevant_shop_meta_keys as $meta_key => $meta_label) {
                if (array_key_exists($meta_key, $values)) {
                    $item->add_meta_data($meta_key, $values[$meta_key]);
                }
            }
        }, 10, 4);

        add_filter('woocommerce_order_item_get_formatted_meta_data', function($meta, $instance) use ($relevant_shop_meta_keys) {

            foreach ($meta as $k => $m) {

                if (isset($relevant_shop_meta_keys[$m->key])) {
                    $meta[$k]->display_key = $relevant_shop_meta_keys[$m->key];
                }
            }

            return $meta;
        }, 10, 2);

        self::$filters_added = true;
    }

    public static function get_products() {

        if (self::$products !== false) {
            return self::$products;
        }

        $products_list = array();

        $query = get_posts(array(
            'post_type'      => array('product'),
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'post_status'    => 'publish',
            'tax_query'      => array(
                array(
                    'taxonomy' => 'product_type',
                    'field'    => 'slug',
                    'terms'    => array('stec_bookable'),
                    'operator' => 'NOT IN'
                )
            ))
        );

        if ($query && is_array($query)) {
            foreach ($query as $id) {
                $products_list[] = wc_get_product($id);
            }
        }

        self::$products = $products_list;

        return self::$products;
    }

    public static function get_products_as_array_list() {

        $arr = array();

        $products = self::get_products();

        foreach ($products as $item) {
            $arr[$item->get_id()] = $item->get_title();
        }

        return $arr;
    }

    /* new code below */

    public static function get_event_products($event_id) {

        $products     = array();
        $products_ids = get_post_meta($event_id, 'products', true);

        foreach ($products_ids as $event_meta_product) {

            $products[] = $event_meta_product->get_parsed_data();
        }

        return $products;
    }

    public static function add_to_cart($item) {

        if (!isset($item['product_id']) || false === Admin_Helper::is_natural_number($item['product_id'])) {
            throw new Stec_Exception(esc_html__('Invalid product', 'stec'));
        }

        if (isset($item['variation_id']) || false !== Admin_Helper::is_natural_number($item['variation_id'])) {
            $item['product_id'] = $item['variation_id'];
        }

        if (!isset($item['quantity']) || false === Admin_Helper::is_natural_number($item['quantity'])) {
            $item['quantity'] = 1;
        }

        if (!isset($item['event_id']) || false === Admin_Helper::is_natural_number($item['event_id'])) {
            throw new Stec_Exception(esc_html__('Invalid event id', 'stec'));
        }

        $event = new Event_Post($item['event_id']);

        if (!$event->get_id()) {
            throw new Stec_Exception(esc_html__('Event does not exist', 'stec'));
        }

        if (!isset($item['repeat_offset']) || false === Admin_Helper::is_natural_number($item['repeat_offset']) || !$event->is_valid_offset($item['repeat_offset'])) {
            throw new Stec_Exception(esc_html__('Invalid event date', 'stec'));
        }

        $event->set_repeat_offset($item['repeat_offset']);

        /**
         * If valid product
         */
        $product = wc_get_product($item['product_id']);

        // Check if exists
        if (!$product || false === $product instanceof \WC_Product) {
            throw new Stec_Exception(esc_html__('This item does not exist', 'stec'));
        }

        // Check in stock / is purchasable
        if (false === $product->is_in_stock() || false === $product->is_purchasable()) {
            throw new Stec_Exception(sprintf(esc_html__('%s is out of stock', 'stec'), $product->get_title()));
        }

        // Check quantity
        if (false === $product->has_enough_stock($item['quantity'])) {
            throw new Stec_Exception(
                    sprintf(esc_html__('We have only %d in stock for %s', 'stec'),
                            $product->get_stock_quantity(),
                            $product->get_title()
                    )
            );
        }

        if ($product->managing_stock()) {
            $products_qty_in_cart = WC()->cart->get_cart_item_quantities();

            if (isset($products_qty_in_cart[$product->get_stock_managed_by_id()]) && !$product->has_enough_stock($products_qty_in_cart[$product->get_stock_managed_by_id()] + $item['quantity'])) {
                throw new Stec_Exception(
                        sprintf(esc_html__('You cannot add that amount of %1$s to the cart &mdash; we have %2$s in stock and you already have %3$s in your cart.', 'stec'),
                                $product->get_title(),
                                wc_format_stock_quantity_for_display($product->get_stock_quantity(), $product),
                                wc_format_stock_quantity_for_display($products_qty_in_cart[$product->get_stock_managed_by_id()], $product))
                );
            }
        }

        try {

            do_action('stec_before_add_to_cart', $product, $item['quantity'], $event);

            $added = WC()->cart->add_to_cart($item['product_id'], $item['quantity']);

            if (false === $added) {
                throw new Stec_Exception(sprintf(esc_html__('Unable to add %s to your cart', 'stec'), $product->get_title()));
            }

            return $added;
        } catch (Exception $ex) {
            throw new Stec_Exception($ex->getMessage());
        }
    }
    
    /**
     * plugins\woocommerce\includes\class-wc-ajax.php get_refreshed_fragments()
     */
    public static function get_cart_fragments() {

        woocommerce_mini_cart();

        $mini_cart = ob_get_clean();

        $data = array(
            'fragments' => apply_filters(
                    'woocommerce_add_to_cart_fragments',
                    array(
                        'div.widget_shopping_cart_content' => '<div class="widget_shopping_cart_content">' . $mini_cart . '</div>',
                    )
            ),
            'cart_hash' => WC()->cart->get_cart_hash(),
        );
        
        return $data;
    }

}
