<?php

namespace Stachethemes\Stec\Elementor;

use \Stachethemes\Stec\Admin_Helper as Admin_Helper;

final class Stec_Elementor {

    const VERSION                   = '1.0.0';
    const MINIMUM_ELEMENTOR_VERSION = '2.9.13';
    const MINIMUM_PHP_VERSION       = '5.4';

    private static $instance = null;

    public static function get_instance() {

        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    protected function __construct() {
        $this->init();
    }

    public function widget_head() {

        if (Admin_Helper::is_elementor_editing_post('stec_esf') || Admin_Helper::is_preview('stec_esf')) {
            \Stachethemes\Stec\Stachethemes_Event_Calendar::get_instance()->load_head();
        }

        if (Admin_Helper::is_elementor_editing_post('stec_sp') || Admin_Helper::is_preview('stec_sp')) {
            \Stachethemes\Stec\Stachethemes_Event_Calendar::get_instance()->load_head();
        }
    }

    public function widget_styles() {

        if (Admin_Helper::is_elementor_editing_post('stec_esf') || Admin_Helper::is_preview('stec_esf')) {
            do_action('stec_submit_form_load_styles');
        }

        if (Admin_Helper::is_elementor_editing_post('stec_sp') || Admin_Helper::is_preview('stec_sp')) {
            do_action('stec_single_load_styles');
        }
    }

    public function widget_scripts() {
        
    }

    public function wpml_widget_translation() {

        if (class_exists('SitePress')) {
            require_once( __DIR__ . '/widgets/single_page/wpml.php' );
            require_once( __DIR__ . '/widgets/event_submit_form/wpml.php' );
        }
    }

    public function init() {

        // Check if Elementor installed and activated
        if (!did_action('elementor/loaded')) {
            add_action('admin_notices', array($this, 'admin_notice_missing_main_plugin'));
            return;
        }

        // Check for required Elementor version
        if (!version_compare(ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=')) {
            add_action('admin_notices', array($this, 'admin_notice_minimum_elementor_version'));
            return;
        }

        // Check for required PHP version
        if (version_compare(PHP_VERSION, self::MINIMUM_PHP_VERSION, '<')) {
            add_action('admin_notices', array($this, 'admin_notice_minimum_php_version'));
            return;
        }

        // Add CPT & WPML tranlsation
        add_action('elementor/init', function() {

            $cpt = array('stec_sp', 'stec_esf');

            foreach ($cpt as $slug) {
                add_post_type_support($slug, 'elementor');
            }

            $this->wpml_widget_translation();
        });

        // Add Plugin actions
        add_action('wp_head', array($this, 'widget_head'));
        add_action('elementor/frontend/after_enqueue_styles', array($this, 'widget_styles'));
        add_action('elementor/frontend/after_enqueue_scripts', array($this, 'widget_scripts'));

        add_action('elementor/widgets/widgets_registered', array($this, 'init_widgets'));
        add_action('elementor/controls/controls_registered', array($this, 'init_controls'));
        add_action('elementor/elements/categories_registered', array($this, 'init_categories'));
    }

    public function init_categories($elements_manager) {

        $elements_manager->add_category(
                'stec',
                array(
                    'title' => esc_html__('Stachethemes Event Calendar', 'stec'),
                    'icon'  => 'far fa-calendar-alt',
                )
        );
    }

    public function admin_notice_missing_main_plugin() {

        if (!Admin_Helper::isset_get('activate')) {
            unset($_GET['activate']);
        }

        $message = sprintf(
                esc_html__('"%1$s" requires "%2$s" to be installed and activated.', 'stec'),
                '<strong>' . esc_html__('Stachethemes Event Calendar Builder', 'stec') . '</strong>',
                '<strong>' . esc_html__('Elementor', 'stec') . '</strong>'
        );

        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function admin_notice_minimum_elementor_version() {

        if (!Admin_Helper::isset_get('activate')) {
            unset($_GET['activate']);
        }

        $message = sprintf(
                esc_html__('"%1$s" requires "%2$s" version %3$s or greater.', 'stec'),
                '<strong>' . esc_html__('Stachethemes Event Calendar Builder', 'stec') . '</strong>',
                '<strong>' . esc_html__('Elementor', 'stec') . '</strong>',
                self::MINIMUM_ELEMENTOR_VERSION
        );

        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function admin_notice_minimum_php_version() {

        if (!Admin_Helper::isset_get('activate')) {
            unset($_GET['activate']);
        }

        $message = sprintf(
                esc_html__('"%1$s" requires "%2$s" version %3$s or greater.', 'stec'),
                '<strong>' . esc_html__('Stachethemes Event Calendar Builder', 'stec') . '</strong>',
                '<strong>' . esc_html__('PHP', 'stec') . '</strong>',
                self::MINIMUM_PHP_VERSION
        );

        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function init_widgets() {

        require_once( __DIR__ . '/widgets/stachethemes_ec.php' );
        require_once( __DIR__ . '/widgets/stec_submit_form.php' );
        require_once( __DIR__ . '/widgets/single_page/elements.php' );
        require_once( __DIR__ . '/widgets/event_submit_form/elements.php' );

        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Widgets\Calendar_Base());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Widgets\Submit_Form());
        
    }

    public function init_controls() {
        
    }

}

$stec_elementor = Stec_Elementor::get_instance();
